@@ -1,2 +1,2 @@
-from .Builder import convert_delimited_file, build_indexes, build_endswith_index
+from .Builder import convert_delimited_file, build_indexes, build_endswith_index, transpose, inner_join
 from .Parser import query, head, tail, get_column_type_from_name, get_num_rows, get_num_cols, NoFilter, StringFilter, FloatFilter, IntFilter, StartsWithFilter, EndsWithFilter, LikeFilter, NotLikeFilter, HeadFilter, TailFilter, AndFilter, OrFilter, FloatRangeFilter, IntRangeFilter, StringRangeFilter
\ No newline at end of file
